import { __decorate } from "tslib";
import { LifecycleMixin } from '@genesislcap/foundation-utils';
import { attr, observable } from '@microsoft/fast-element';
import { FoundationElement } from '@microsoft/fast-foundation';
import { ChartDatasource } from './chart-datasource';
import { g2plotChartTemplate as template } from './chart.template';
import { LineChart, AreaChart, BarChart, ColumnChart, DonutChart, PieChart, DualAxesChart, RoseChart, ScatterChart, StockChart, MixChart, } from './models';
import { registerRapidShape } from './settings';
ChartDatasource;
export class G2PlotChart extends LifecycleMixin(FoundationElement) {
    constructor() {
        super(...arguments);
        this.type = 'bar';
        this.legendSelected = {};
        this.initalizeChart = {
            line: (config) => (this.chart = LineChart(this.chartContainer, config)),
            area: (config) => (this.chart = AreaChart(this.chartContainer, config)),
            bar: (config) => (this.chart = BarChart(this.chartContainer, config)),
            rose: (config) => (this.chart = RoseChart(this.chartContainer, config)),
            scatter: (config) => (this.chart = ScatterChart(this.chartContainer, config)),
            pie: (config) => (this.chart = PieChart(this.chartContainer, config)),
            donut: (config) => (this.chart = DonutChart(this.chartContainer, config)),
            column: (config) => (this.chart = ColumnChart(this.chartContainer, config)),
            stock: (config) => (this.chart = StockChart(this.chartContainer, config)),
            dualaxes: (config) => (this.chart = DualAxesChart(this.chartContainer, config)),
            mix: (config) => (this.chart = MixChart(this.chartContainer, config)),
        };
        this.getLegendNameFromData = {
            pie: ({ groupBy }) => groupBy,
            donut: ({ groupBy }) => groupBy,
            bar: ({ series }) => series,
            rose: ({ series }) => series,
        };
        this.initalizeConfiguration = () => {
            var _a, _b;
            const configuration = (_a = this.config) !== null && _a !== void 0 ? _a : {};
            configuration.data = (_b = this.data) !== null && _b !== void 0 ? _b : [[], []];
            configuration.legend = Object.assign(Object.assign({}, configuration.legend), { selected: Object.assign({}, this.legendSelected) });
            return configuration;
        };
    }
    dataChanged(oldValue, newValue) {
        if (!newValue)
            return;
        if (!this.chart && this.isConnected) {
            this.renderChart();
        }
        if (oldValue && oldValue !== newValue) {
            this.chart.changeData(this.data);
        }
    }
    deepClone() {
        const copy = super.deepClone();
        copy.config = structuredClone(this.config);
        copy.data = structuredClone(this.data);
        if (this.legendParser) {
            copy.legendParser = this.legendParser.bind({});
        }
        return copy;
    }
    renderChart() {
        this.initalizeChart[this.type](this.initalizeConfiguration());
        this.chart.render();
    }
    connectedCallback() {
        var _a;
        super.connectedCallback();
        registerRapidShape();
        if (this.data || (this.type == 'mix' && this.config)) {
            this.renderChart();
        }
        this.cacheHiddenLegendItemsCallback = this.cacheHiddenLegendItemsCallback.bind(this);
        // as the chart is destroyed in the disconnect the event is wiped
        (_a = this.chart) === null || _a === void 0 ? void 0 : _a.on('legend-item:click', this.cacheHiddenLegendItemsCallback);
    }
    disconnectedCallback() {
        this.chart.destroy();
    }
    /**
     * Get which legend items are selected (shown/hidden) so they can be re-applied during a subsequent connectedCallback
     *
     * @remarks
     * When moving a chart around with `appendChild()` the state of hidden/shown fields is reset. This callback function
     * will cache the state of the legend items so they can be re-applied when the chart is re-rendered. The cache
     * is applied in `this.initalizeConfiguration()`.
     *
     * To get the state of the legend items, the chart's `filteredData` is used. This is the data that is currently
     * shown on the chart. If we diff this with all of the data from `this.data` we can work out which fields are hidden or shown.
     *
     * To get the legend name from the data to pass to the options we use a legend parser which is a function that takes a single
     * data item and extracts the legend name from it. There are defaults for certain chart types in `this.getLegendNameFromData`
     * but this can be overridden by passing a `legendParser` function to the chart.
     */
    cacheHiddenLegendItemsCallback(...args) {
        if (!(this.type in this.getLegendNameFromData || typeof this.legendParser === 'function')) {
            return;
        }
        const getLegendNameFromData = typeof this.legendParser === 'function'
            ? this.legendParser
            : this.getLegendNameFromData[this.type];
        const displayedSet = new Set(args[0].view.filteredData.map(getLegendNameFromData));
        this.legendSelected = this.data.map(getLegendNameFromData).reduce((acum, legenedName) => (Object.assign(Object.assign({}, acum), { [`${legenedName}`]: displayedSet.has(legenedName) })), {});
        const hasHiddenFields = Object.values(this.legendSelected).includes(false);
        const title = hasHiddenFields ? '+' : '';
        // Not importing types so foundation-marketplace doesn't have a dependency on foundation-layout
        this.$emit('change-title', { title, mode: 'suffix' });
    }
}
__decorate([
    attr
], G2PlotChart.prototype, "type", void 0);
__decorate([
    observable
], G2PlotChart.prototype, "config", void 0);
__decorate([
    observable
], G2PlotChart.prototype, "data", void 0);
// explicitly declaring for extensibility
export const foundationChartShadowOptions = undefined;
export const defaultChartConfig = {};
/**
 * The G2plot Chart
 *
 * @public
 * @remarks
 * HTML Element: \<design-system-g2plot-chart\>
 */
export const g2plotChart = G2PlotChart.compose(Object.assign({ baseName: 'g2plot-chart', template, shadowOptions: foundationChartShadowOptions }, defaultChartConfig));
